/**************************************************************************************************
Copyright (C) 2016 Andrey Tsyganov, UlSPU. All rights reserved.

Contact information
-------------------
Andrey Tsyganov, Ulyanovsk State Pedagogical University, Laboratory of Mathematical Modeling
Web    :  http://lmm.ulspu.ru
e-mail :  andrew.tsyganov@gmail.com
**************************************************************************************************/


#include <conio.h>
#include <iomanip>
#include "../../src/ga/ga_seq.h"


const char goal[] = "Hello, world!"; // String to guess
const int  len = 13; // String length (without null character)
const char max_bias = 5; // Maximum bias of the changed symbol


class Indiv
{
public:
	Indiv()
	{
		data_[len] = '\0';
	}

	friend class Hello_GA_SEQ;
	friend std::ostream& operator<<(std::ostream& os, const Indiv& indiv)
	{
		os << indiv.data_;

		return os;
	}

private:
	char data_[len + 1];
};


class Hello_GA_SEQ: public ga::GA_SEQ< int, Indiv >
{
public:
	virtual void output(const ga::GA_Param<int>& param, const ga::GA_State<int, Indiv>& state, std::ostream& os);
	virtual int fitness(const Indiv& indiv, const ga::GA_State<int, Indiv>& state); // Evaluate fitness
	virtual Indiv generate(const ga::GA_State<int, Indiv>& state, ga::RndGen& rnd); // Generate individual
	virtual Indiv crossover(const Indiv& parent1, const Indiv& parent2, const ga::GA_State<int, Indiv>& state, ga::RndGen& rnd); // Perform crossover
	virtual Indiv mutation(const Indiv& indiv, const ga::GA_State<int, Indiv>& state, ga::RndGen& rnd); // Perform mutation 
};


void Hello_GA_SEQ::output(const ga::GA_Param<int>& param, const ga::GA_State<int, Indiv>& state, std::ostream& os)
{
	if (state.gen_ == 0)
		os << state << "\n";
	else
	{
		/*
		os << "gen_: " << state.gen_ << "\n";
		os << "population_:\n";
		for (size_t i = 0; i < state.population_.size(); i++)
			os << std::setw(2) << i << ") " << state.population_[i] << " fitness_: " << state.fitnesses_[i] << "\n";
		*/
		os << "gen_: " << state.gen_ << " population_[0]: " << state.population_[0] << " fitnesses_[0]: " << state.fitnesses_[0] << "\n";
	}
}


int Hello_GA_SEQ::fitness(const Indiv& indiv, const ga::GA_State<int, Indiv>& state)
{
	int sum = 0;

	for (int i = 0; i < len; i++)
		sum += abs(indiv.data_[i] - goal[i]);

	return sum;
}


Indiv Hello_GA_SEQ::generate(const ga::GA_State<int, Indiv>& state, ga::RndGen& rnd)
{
	Indiv indiv;

	for (int i = 0; i < len; i++)
		indiv.data_[i] = (char)rnd.IRandom(32, 127);

	return indiv;
}


Indiv Hello_GA_SEQ::crossover(const Indiv& parent1, const Indiv& parent2, const ga::GA_State<int, Indiv>& state, ga::RndGen& rnd)
{
	// Scattered crossover
	Indiv child;

	child.data_[len] = '\0';
	for (int i = 0; i < len; i++)
		// Better (first) parent contributes two times more genes
		child.data_[i] = !(rnd.IRandom(0, 2)) ? parent2.data_[i] : parent1.data_[i];

	return child;
}


Indiv Hello_GA_SEQ::mutation(const Indiv& indiv, const ga::GA_State<int, Indiv>& state, ga::RndGen& rnd)
{
	Indiv child = indiv;

	int pos = rnd.IRandom(0, len - 1);

	child.data_[pos] += (char)rnd.IRandom(-max_bias, max_bias);

	if (child.data_[pos] < 32)
		child.data_[pos] = 32;

	return child;
}


int main()
{
	ga::GA_Param<int> param;
	ga::GA_Result <int, Indiv> res;
	std::vector<Indiv> v_init_sol;
	Hello_GA_SEQ ga;

	double start_time, end_time;

	param.setGoal(ga::MIN);
	param.setPopSize(15);
	param.setEliteCount(1);
	param.setDeadCount(1);
	param.setTournamentCount(2);
	param.setCrossoverFrac(0.6);

	//param.setStopCond(ga::GEN_LIM);
	//param.setGenLim(300);

	param.setStopCond(ga::FITNESS_LIM);
	param.setFitnessLim(0);

	//param.setStopCond(ga::TIME_LIM);
	//param.setTimeLim(0.005);

	param.setOfname("log.txt");

	std::cout << param << "\n";
	std::cout << "Start GA...\n";

	start_time = get_wall_time();
	ga.run(param, v_init_sol, res);
	end_time = get_wall_time();

	std::cout << "End GA\n";
	std::cout << "Elapsed time: " << end_time - start_time << " second(s)\n\n";
	std::cout << res << "\n";

	_getch();

	return 0;
}
