/**************************************************************************************************
Copyright (C) 2016 Andrey Tsyganov, UlSPU. All rights reserved.

Contact information
-------------------
Andrey Tsyganov, Ulyanovsk State Pedagogical University, Laboratory of Mathematical Modeling
Web    :  http://lmm.ulspu.ru
e-mail :  andrew.tsyganov@gmail.com
**************************************************************************************************/


#include <conio.h>
#include "../../src/sa/sa_seq.h"


const char goal[] = "Hello, world!"; // String to guess
const int  len = 13; // String length (without null character)
const char max_bias = 5; // Maximum bias of the changed symbol


class Solution
{
public:
	Solution()
	{
		data_[len] = '\0';
	}

	friend class Hello_SA_SEQ;
	friend std::ostream& operator<<(std::ostream& os, const Solution& solution)
	{
		os << solution.data_;

		return os;
	}

private:
	char data_[len + 1];
};


class Hello_SA_SEQ: public sa::SA_SEQ< int, Solution >
{
public:
	virtual void output(const sa::SA_Param<int>& param, const sa::SA_State<int, Solution>& state, std::ostream& os);
	virtual int cost(const Solution& solution, const sa::SA_State<int, Solution>& state);
	virtual Solution generate(const sa::SA_State<int, Solution>& state, sa::RndGen& rnd);
	virtual Solution neighbour(const Solution& solution, const sa::SA_State<int, Solution>& state, sa::RndGen& rnd);
};


void Hello_SA_SEQ::output(const sa::SA_Param<int>& param, const sa::SA_State<int, Solution>& state, std::ostream& os)
{
	if (state.step_ == 0)
		os << "state:\n" << state << "\n";
	else
		os << "step_: " << state.step_ << " step_iter_: " << state.step_iter_ << " best_: " << state.best_ << " best_cost_: " << state.best_cost_ << " stall_step_: " << state.stall_step_ << "\n";
}


int Hello_SA_SEQ::cost(const Solution& solution, const sa::SA_State<int, Solution>& state)
{
	int sum = 0;

	for (int i = 0; i < len; i++)
		sum += abs(solution.data_[i] - goal[i]);

	return sum;
}


Solution Hello_SA_SEQ::generate(const sa::SA_State<int, Solution>& state, sa::RndGen& rnd)
{
	Solution solution;

	for (int i = 0; i < len; i++)
		solution.data_[i] = (char)rnd.IRandom(32, 127);

	return solution;
}


Solution Hello_SA_SEQ::neighbour(const Solution& solution, const sa::SA_State<int, Solution>& state, sa::RndGen& rnd)
{
	Solution neighb = solution;

	int pos = rnd.IRandom(0, len - 1);

	neighb.data_[pos] += (char)rnd.IRandom(-max_bias, max_bias);

	if (neighb.data_[pos] < 32)
		neighb.data_[pos] = 32;

	return neighb;
}


int main()
{
	sa::SA_Param<int> param;
	sa::SA_Result <int, Solution> res;
	std::vector<Solution> v_init_sol;
	Hello_SA_SEQ sa;

	double start_time, end_time;

	param.setGoal(sa::MIN);
	param.setStepLen(5);

	param.setStopCond(sa::COST_LIM);
	param.setCostLim(0);

	//param.setStopCond(sa::STEPS_LIM);
	//param.setStepsLim(100);

	//param.setStopCond(sa::STALL_STEPS_LIM);
	//param.setStallStepsLim(50);

	//param.setStopCond(sa::TIME_LIM);
	//param.setTimeLim(1.5);

	//param.setStopCond(sa::TEMP_LIM);
	//param.setTempLim(0.1);

	param.setCooling(sa::EXP);

	param.setOfname("log.txt");

	std::cout << param << "\n";
	std::cout << "Start SA...\n";

	start_time = get_wall_time();
	sa.run(param, v_init_sol, res);
	end_time = get_wall_time();

	std::cout << "End SA\n";
	std::cout << "Elapsed time: " << end_time - start_time << " second(s)\n\n";
	std::cout << res << "\n";

	_getch();

	return 0;
}