/**************************************************************************************************
Copyright (C) 2016 Andrey Tsyganov, UlSPU. All rights reserved.

Contact information
-------------------
Andrey Tsyganov, Ulyanovsk State Pedagogical University, Laboratory of Mathematical Modeling
Web    :  http://lmm.ulspu.ru
e-mail :  andrew.tsyganov@gmail.com
**************************************************************************************************/


#include <conio.h>
#include "../../src/shc/shc_omp.h"



const char goal[] = "Hello, world!"; // String to guess
const int  len = 13; // String length (without null character)
const char max_bias = 5; // Maximum bias of the changed symbol


class Solution
{
public:
	Solution()
	{
		data_[len] = '\0';
	}

	friend class Hello_SHC_OMP;
	friend std::ostream& operator<<(std::ostream& os, const Solution& solution)
	{
		os << solution.data_;

		return os;
	}

private:
	char data_[len + 1];
};


class Hello_SHC_OMP: public shc::SHC_OMP< int, Solution >
{
public:
	virtual void output(const shc::SHC_Param<int>& param, const shc::SHC_State<int, Solution>& state, std::ostream& os);
	virtual int cost(const Solution& solution, const shc::SHC_State<int, Solution>& state);
	virtual Solution generate(const shc::SHC_State<int, Solution>& state, shc::RndGen& rnd);
	virtual Solution neighbour(const Solution& solution, const shc::SHC_State<int, Solution>& state, shc::RndGen& rnd);
};


void Hello_SHC_OMP::output(const shc::SHC_Param<int>& param, const shc::SHC_State<int, Solution>& state, std::ostream& os)
{
	if (state.step_ == 0)
		os << state << "\n";
	else
		os << "step_: " << state.step_ << " stall_step_: " << state.stall_step_ << " best_:  " << state.best_ << " best_cost_: " << state.best_cost_ << "\n";
}


int Hello_SHC_OMP::cost(const Solution& solution, const shc::SHC_State<int, Solution>& state)
{
	int sum = 0;

	for (int i = 0; i < len; i++)
		sum += abs(solution.data_[i] - goal[i]);

	return sum;
}


Solution Hello_SHC_OMP::generate(const shc::SHC_State<int, Solution>& state, shc::RndGen& rnd)
{
	Solution solution;

	for (int i = 0; i < len; i++)
		solution.data_[i] = (char)rnd.IRandom(32, 127);

	return solution;
}


Solution Hello_SHC_OMP::neighbour(const Solution& solution, const shc::SHC_State<int, Solution>& state, shc::RndGen& rnd)
{
	Solution neighb = solution;

	int pos = rnd.IRandom(0, len - 1);

	neighb.data_[pos] += (char)rnd.IRandom(-max_bias, max_bias);

	if (neighb.data_[pos] < 32)
		neighb.data_[pos] = 32;

	return neighb;
}


int main()
{
	shc::SHC_Param<int> param;
	shc::SHC_Result <int, Solution> res;
	std::vector<Solution> v_init_sol;
	Hello_SHC_OMP shc;

	double start_time, end_time;

	param.setGoal(shc::MIN);

	//param.setStopCond(shc::STEPS_LIM);
	//param.setStepsLim(1000);

	//param.setStopCond(shc::STALL_STEPS_LIM);
	//param.setStallStepsLim(100);

	param.setStopCond(shc::COST_LIM);
	param.setCostLim(0);

	//param.setStopCond(shc::TIME_LIM);
	//param.setTimeLim(1.5);

	param.setCommRate(200);
	param.setStopPolicy(shc::ONE);

	param.setOfname("log.txt");

	std::cout << param << "\n";
	std::cout << "Start SHC...\n";

	start_time = get_wall_time();
	shc.run(param, v_init_sol, res);
	end_time = get_wall_time();

	std::cout << "End SHC\n";
	std::cout << "Elapsed time: " << end_time - start_time << " second(s)\n\n";
	std::cout << res << "\n";

	_getch();

	return 0;
}