/**************************************************************************************************
Copyright (C) 2016 Andrey Tsyganov, UlSPU. All rights reserved.

Contact information
-------------------
Andrey Tsyganov, Ulyanovsk State Pedagogical University, Laboratory of Mathematical Modeling
Web    :  http://lmm.ulspu.ru
e-mail :  andrew.tsyganov@gmail.com
**************************************************************************************************/


#ifndef UTILS_H
#define UTILS_H


#include <string>
#include <ostream>
#include <fstream>
#include <iostream>
#include <algorithm>


size_t Width(size_t val)
{
    size_t w = 0;
    for (size_t n = val; n > 0; n /= 10)
        ++w;

    return w;
}


std::string IntToString(size_t val)
{
    char buff[32];
    sprintf_s(buff, "%zu", val);

    return std::string(buff);
}


std::string IntToStringZ(size_t val, size_t max)
{
    char buff[32];
	sprintf_s(buff, "%0*u", Width(max), val);

    return std::string(buff);
}


std::string ToLower(const std::string& s)
{
    std::string str = s;
    transform(str.begin(), str.end(), str.begin(), tolower);

    return str;
}


std::string ToLower(const char* s)
{
    std::string str(s);
    transform(str.begin(), str.end(), str.begin(), tolower);

    return str;
}


std::string GetFileExtension(const std::string& filename)
{
    size_t pos = filename.rfind('.');

    if (pos == filename.npos)
        return ""; 
    else
        return filename.substr(pos+1);
}


std::string CreatePath(const std::string& dir, const std::string& file)
{
    size_t n = dir.length();

	if (n == 0)
		return file;

    if ((dir[n-1] == '\\') || (dir[n-1] == '/'))
        return dir + file;

    size_t pos = dir.find_last_of("/\\");

    if (pos == dir.npos)
        return dir + "/" + file;

    if (dir[pos] == '\\')
        return dir + "\\" + file;
    else
        return dir + "/" + file;
}


void SplitFileName(const std::string& path, std::string& dir, std::string& file)
{
    size_t pos = path.find_last_of("/\\");

	if (pos == path.npos)
	{
		dir = "";
		file = path;
	}
	else
	{
		dir = path.substr(0, pos);
		file = path.substr(pos + 1);
	}
}


void SplitFileName(const std::string& path, std::string& dir, std::string& name, std::string& ext)
{
	SplitFileName(path, dir, name);
    
    size_t pos = name.find_last_of(".");
    
    if (pos == name.npos)
        ext = "";
    else
    {
        ext  = name.substr(pos+1);
        name = name.substr(0, pos);
    }
}

std::streambuf *buf;


std::streambuf* OSBuf(const std::string& ofname, std::ofstream& of, int rank, int nthreads)
{
	// Obtain std::ostream either from std::cout or std::ofstream(file)
	// see solution of Johannes Schaub (aka litb)
	// at http://stackoverflow.com/questions/366955/obtain-a-stdostream-either-from-stdcout-or-stdofstreamfile

	if (ofname.empty())
		return std::cout.rdbuf();
	else
	{
		std::string path;

		if (nthreads > 1)
		{
			std::string dir, name, ext, fname;

			SplitFileName(ofname, dir, name, ext);
			fname = name + IntToStringZ(rank, nthreads) + "." + ext;
			path = CreatePath(dir, fname);
		}
		else
			path = ofname;

		of.open(path.c_str());

		return of.rdbuf();
	}
}


#endif
